module MetaBuild

  class App

    require 'fileutils'
    require 'zip'
    require 'json'
    require 'tmpdir'

    require 'meta_build/version'
    require 'meta_build/exceptions'
    require 'meta_build/helper/app_helper'
    require 'meta_build/helper/properties_helper'
    require 'meta_build/helper/zip_helper'
    require 'meta_build/extractor/base_extractor'
    require 'meta_build/extractor/ear_extractor'
    require 'meta_build/extractor/war_extractor'
    require 'meta_build/extractor/jar_extractor'
    require 'meta_build/extractor/extractor_factory'
    require 'meta_build/parser/base_parser'
    require 'meta_build/parser/ear_parser'
    require 'meta_build/parser/war_parser'
    require 'meta_build/parser/jar_parser'
    require 'meta_build/parser/parser_factory'
    require 'meta_build/builder/meta_builder'

    attr_reader :compressed_file, :output_dir, :name, :replace_files, :create_dir

    def initialize(options = {})
      message = MetaBuild::Helper::AppHelper.validate_options options
      if message
        raise "[ERROR] #{message}"
      end

      @compressed_file = options[:compressed_file]
      @output_dir = options[:output_dir]
      @name = MetaBuild::Helper::AppHelper.set_target_name(options[:name], options[:compressed_file])
      @replace_files = options[:replace_files] ||= false
      @create_dir = options[:create_dir] ||= false
    end

    def extract_metadata
      _configure_prerequisites

      hash = MetaBuild::Builder::MetaBuilder.build file: @compressed_file
      
      FileUtils.makedirs @output_dir unless File.exist? @output_dir
      target_file = File.join(@output_dir, @name)

      if File.exist?(target_file) && (!@replace_files)
        puts "[ERROR] The file #{target_file} already exist. You might want to call this feature with the parameter [-r, --replace-file]"
        exit 1
      end

      File.open(target_file, 'w') { |io| io.write hash.to_json }

      FileUtils.remove_dir MetaBuild::Helper::AppHelper.work_dir, true
    end

    private
      def _configure_prerequisites
        _notify_create_dir
        _notify_truncate_dir
      end

      def _notify_create_dir
        return unless @create_dir
        FileUtils.remove_dir @output_dir, true
      end

      def _notify_truncate_dir
        return unless @replace_files
        FileUtils.remove Dir.glob("#{@output_dir}/**/*")
      end

  end

end